import math
from Vector import angle_vector, Vector
class Object(object):
    '''Class to represent any game object.

    An object can have health, which can be depleted until it is destroyed.

    An object can be static; static objects do not collide with other static
    objects, but are not fixed in place.'''
    bulletexplodesound = "boom.wav"
    bulletsound = "shot.wav"
    def __init__(self, health=None, static=None, diesound=None,
                 angle=0, remains=None, corpse=None):
        self.health=health
        self.static=static
        self.diesound=diesound
        self.angle=angle
        self.remains=remains
        self.corpse=corpse
        self.dead = False
        self.extras = {}

    def thread(self):
        while True:
            yield None

    def passable(self, part, o2, n, t, game):
        return False

    def pre_collide(self, part, pass1, o2, pass2, n, t, game):
        pass

    def post_collide(self, part, pass1, o2, pass2, n, t, game):
        pass

    def get_elasticity(self, part):
        return 0.9

    def die(self, game):
        self.dead = True

class Powerup(Object):
    def __init__(self, **kwargs):
        super(Powerup, self).__init__(**kwargs)

    def add_geometry(self, geometry):
        geometry.add_round(self, 0, 10)
        
    def passable(self, part, o2):
        return True

    def pre_collide(self, o2, game, **kwargs):
        if isinstance(o2, Player):
            game.delete_object(self)

class Player(Object):
    bulletexplodeimage = 'bullethit'
    bulletimage = 'bullet'
    bulletsound = "electric.wav"
    bulletexplodesound = "zap.wav"
    killstr = "human"
    attackimg = None
    def __init__(self, size, bulletspeed=None, killstr=None, **kwargs):
        kwargs.setdefault('health',100)
        kwargs.setdefault('angle', 0)
        super(Player, self).__init__(**kwargs)#Geometry([Round(16)]), pos)
        if killstr: self.killstr = killstr
        self.size = size
        self.bulletspeed = bulletspeed
        #self.hooked = False
        self.bulletpos = None

    def bullet_pos(self, pos, bulletsize):
##        print pos, self.size, self.angle, angle_vector(self.angle)*(self.size+bulletsize)
        return pos + (angle_vector(self.angle)*(self.size+bulletsize+1))


class Enemy(Object):
    bulletexplodeimage = 'explosion'
    bulletimage = 'bullet'
    killstr = "enemy"
    def __init__(self, ai=None, size=None, bulletpos=None, health=None,
                 angle=None, bulletspeed = None, bulletexplodeimage=None,
                 bulletimage=None, bulletsize=None, killstr=None, **kwargs):
        kwargs.setdefault('health',health)
        super(Enemy, self).__init__(**kwargs)#Geometry([Round(16)]), pos)
        self.angle = angle
        self.size = size
        self.ai = ai
        self.bulletpos = bulletpos
        self.bulletspeed = bulletspeed
        self.bulletsize = bulletsize
        if killstr: self.killstr = killstr
        if bulletimage:
            self.bulletimage = bulletimage
        if bulletexplodeimage:
            self.bulletexplodeimage = bulletexplodeimage

class Switch(Object):
    def __init__(self, action=None, **kwargs):
        kwargs.setdefault('static', True)
        super(Switch, self).__init__(**kwargs)
        self.action = action # action is a Pattern, or has a thread() method

    def pre_collide(self, part, pass1, o2, pass2, n, t, game):
        game.add_action(self.action.thread(game, self, self))
